library angular2.src.core.compiler.compiler;
import 'package:angular2/src/facade/lang.dart' show Type, isBlank, isPresent, BaseException, normalizeBlank, stringify;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper, Map, MapWrapper;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import 'package:angular2/change_detection.dart' show ChangeDetection, Parser;
import './directive_metadata_reader.dart' show DirectiveMetadataReader;
import './view.dart' show ProtoView;
import './pipeline/compile_pipeline.dart' show CompilePipeline;
import './pipeline/compile_element.dart' show CompileElement;
import './pipeline/default_steps.dart' show createDefaultSteps;
import './template_loader.dart' show TemplateLoader;
import './directive_metadata.dart' show DirectiveMetadata;
import '../annotations/annotations.dart' show Component;
import './shadow_dom_emulation/content_tag.dart' show Content;
import './shadow_dom_strategy.dart' show ShadowDomStrategy;
class CompilerCache {
Map _cache;
CompilerCache() {
  this._cache = MapWrapper.create();
}
set(Type component, ProtoView protoView) {
  MapWrapper.set(this._cache, component, protoView);
}
ProtoView get(Type component) {
  var result = MapWrapper.get(this._cache, component);
  return normalizeBlank(result);
}
clear() {
  MapWrapper.clear(this._cache);
}
}
class Compiler {
DirectiveMetadataReader _reader;
Parser _parser;
CompilerCache _compilerCache;
ChangeDetection _changeDetection;
TemplateLoader _templateLoader;
Map _compiling;
ShadowDomStrategy _shadowDomStrategy;
List _shadowDomDirectives;
Compiler(ChangeDetection changeDetection, TemplateLoader templateLoader, DirectiveMetadataReader reader, Parser parser, CompilerCache cache, ShadowDomStrategy shadowDomStrategy) {
  this._changeDetection = changeDetection;
  this._reader = reader;
  this._parser = parser;
  this._compilerCache = cache;
  this._templateLoader = templateLoader;
  this._compiling = MapWrapper.create();
  this._shadowDomStrategy = shadowDomStrategy;
  this._shadowDomDirectives = [];
  var types = shadowDomStrategy.polyfillDirectives();
  for (var i = 0; i < types.length; i++) {
    ListWrapper.push(this._shadowDomDirectives, reader.read(types[i]));
  }
}
List createSteps(DirectiveMetadata component) {
  var directives = [];
  var cmpDirectives = ListWrapper.map(component.componentDirectives, (d) {
    return this._reader.read(d);
  });
  directives = ListWrapper.concat(directives, cmpDirectives);
  directives = ListWrapper.concat(directives, this._shadowDomDirectives);
  return createDefaultSteps(this._changeDetection, this._parser, component, directives, this._shadowDomStrategy);
}
Future compile(Type component, [Element templateRoot = null]) {
  return this._compile(this._reader.read(component), templateRoot);
}
_compile(DirectiveMetadata cmpMetadata, [Element templateRoot = null]) {
  var pvCached = this._compilerCache.get(cmpMetadata.type);
  if (isPresent(pvCached)) {
    return PromiseWrapper.resolve(pvCached);
  }
  var pvPromise = MapWrapper.get(this._compiling, cmpMetadata.type);
  if (isPresent(pvPromise)) {
    return pvPromise;
  }
  var tplPromise = isBlank(templateRoot) ? this._templateLoader.load(cmpMetadata) : PromiseWrapper.resolve(templateRoot);
  pvPromise = PromiseWrapper.then(tplPromise, (el) {
    return this._compileTemplate(el, cmpMetadata);
  }, (_) {
    throw new BaseException('''Failed to load the template for ${stringify(cmpMetadata.type)}''');
  });
  MapWrapper.set(this._compiling, cmpMetadata.type, pvPromise);
  return pvPromise;
}
Future _compileTemplate(Element template, cmpMetadata) {
  this._shadowDomStrategy.processTemplate(template, cmpMetadata);
  var pipeline = new CompilePipeline(this.createSteps(cmpMetadata));
  var compileElements = pipeline.process(template);
  var protoView = compileElements[0].inheritedProtoView;
  this._compilerCache.set(cmpMetadata.type, protoView);
  MapWrapper.delete(this._compiling, cmpMetadata.type);
  var componentPromises = [];
  for (var i = 0; i < compileElements.length; i++) {
    var ce = compileElements[i];
    if (isPresent(ce.componentDirective)) {
      var componentPromise = this._compileNestedProtoView(ce);
      ListWrapper.push(componentPromises, componentPromise);
    }
  }
  return PromiseWrapper.then(PromiseWrapper.all(componentPromises), (_) {
    return protoView;
  }, (e) {
    throw new BaseException('''${e} -> Failed to compile ${stringify(cmpMetadata.type)}''');
  });
}
Future _compileNestedProtoView(CompileElement ce) {
  var pvPromise = this._compile(ce.componentDirective);
  pvPromise.then((protoView) {
    ce.inheritedElementBinder.nestedProtoView = protoView;
  });
  return pvPromise;
}
}
