library angular2.test.core.compiler.compiler_spec;
import 'package:angular2/test_lib.dart' show describe, beforeEach, it, expect, ddescribe, iit, el, IS_DARTIUM;
import 'package:angular2/src/facade/dom.dart' show DOM, Element, TemplateElement;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper, Map, MapWrapper;
import 'package:angular2/src/facade/lang.dart' show Type, isBlank;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/core/compiler/compiler.dart' show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/view.dart' show ProtoView;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/compiler/pipeline/compile_element.dart' show CompileElement;
import 'package:angular2/src/core/compiler/pipeline/compile_step.dart' show CompileStep;
import 'package:angular2/src/core/compiler/pipeline/compile_control.dart' show CompileControl;
import 'package:angular2/src/core/compiler/template_loader.dart' show TemplateLoader;
import 'package:angular2/change_detection.dart' show Lexer, Parser, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show ShadowDomStrategy, NativeShadowDomStrategy;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;
main() {
  describe('compiler', () {
    var reader;
    beforeEach(() {
      reader = new DirectiveMetadataReader();
    });
    createCompiler(processClosure, [ShadowDomStrategy strategy = null, XHRMock xhr = null]) {
      var steps = [new MockStep(processClosure)];
      if (isBlank(strategy)) {
        strategy = new NativeShadowDomStrategy();
      }
      if (isBlank(xhr)) {
        xhr = new XHRMock();
      }
      return new TestableCompiler(reader, steps, strategy, xhr);
    }
    it('should run the steps and return the ProtoView of the root element', (done) {
      var rootProtoView = new ProtoView(null, null, null);
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = rootProtoView;
      });
      compiler.compile(MainComponent, el('<div></div>')).then((protoView) {
        expect(protoView).toBe(rootProtoView);
        done();
      });
    });
    it('should use the given element', (done) {
      var element = el('<div></div>');
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = new ProtoView(current.element, null, null);
      });
      compiler.compile(MainComponent, element).then((protoView) {
        expect(protoView.element).toBe(element);
        done();
      });
    });
    it('should use the inline template if no element is given explicitly', (done) {
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = new ProtoView(current.element, null, null);
      });
      compiler.compile(MainComponent, null).then((protoView) {
        expect(DOM.getInnerHTML(protoView.element)).toEqual('inline component');
        done();
      });
    });
    it('should use the shadow dom strategy to process the template', (done) {
      if (IS_DARTIUM) {
        done();
        return ;
      }
      var templateHtml = 'processed template';
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = new ProtoView(current.element, null, null);
      }, new FakeShadowDomStrategy(templateHtml));
      compiler.compile(MainComponent, null).then((protoView) {
        expect(DOM.getInnerHTML(protoView.element)).toEqual('processed template');
        done();
      });
    });
    it('should load nested components', (done) {
      var mainEl = el('<div></div>');
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = new ProtoView(current.element, null, null);
        current.inheritedElementBinder = current.inheritedProtoView.bindElement(null);
        if (identical(current.element, mainEl)) {
          current.componentDirective = reader.read(NestedComponent);
        }
      });
      compiler.compile(MainComponent, mainEl).then((protoView) {
        var nestedView = protoView.elementBinders[0].nestedProtoView;
        expect(DOM.getInnerHTML(nestedView.element)).toEqual('nested component');
        done();
      });
    });
    it('should cache compiled components', (done) {
      var element = el('<div></div>');
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = new ProtoView(current.element, null, null);
      });
      var firstProtoView;
      compiler.compile(MainComponent, element).then((protoView) {
        firstProtoView = protoView;
        return compiler.compile(MainComponent, element);
      }).then((protoView) {
        expect(firstProtoView).toBe(protoView);
        done();
      });
    });
    it('should re-use components being compiled', (done) {
      var nestedElBinders = [];
      var mainEl = el('<div><div class="nested"></div><div class="nested"></div></div>');
      var compiler = createCompiler((parent, current, control) {
        if (DOM.hasClass(current.element, 'nested')) {
          current.inheritedProtoView = new ProtoView(current.element, null, null);
          current.inheritedElementBinder = current.inheritedProtoView.bindElement(null);
          current.componentDirective = reader.read(NestedComponent);
          ListWrapper.push(nestedElBinders, current.inheritedElementBinder);
        }
      });
      compiler.compile(MainComponent, mainEl).then((protoView) {
        expect(nestedElBinders[0].nestedProtoView).toBe(nestedElBinders[1].nestedProtoView);
        done();
      });
    });
    it('should allow recursive components', (done) {
      var compiler = createCompiler((parent, current, control) {
        current.inheritedProtoView = new ProtoView(current.element, null, null);
        current.inheritedElementBinder = current.inheritedProtoView.bindElement(null);
        current.componentDirective = reader.read(RecursiveComponent);
      });
      compiler.compile(RecursiveComponent, null).then((protoView) {
        expect(protoView.elementBinders[0].nestedProtoView).toBe(protoView);
        done();
      });
    });
    describe('XHR', () {
      it('should load template via xhr', (done) {
        var xhr = new XHRMock();
        xhr.expect('/parent', 'xhr');
        var compiler = createCompiler((parent, current, control) {
          current.inheritedProtoView = new ProtoView(current.element, null, null);
        }, null, xhr);
        compiler.compile(XHRParentComponent).then((protoView) {
          expect(DOM.getInnerHTML(protoView.element)).toEqual('xhr');
          done();
        });
        xhr.flush();
      });
      it('should return a rejected promise when loading a template fails', (done) {
        var xhr = new XHRMock();
        xhr.expect('/parent', null);
        var compiler = createCompiler((parent, current, control) {}, null, xhr);
        PromiseWrapper.then(compiler.compile(XHRParentComponent), (_) {
          throw 'Failure expected';
        }, (e) {
          expect(e.message).toEqual('Failed to load the template for XHRParentComponent');
          done();
        });
        xhr.flush();
      });
    });
  });
}
@Component(template: const TemplateConfig(url: '/parent')) 
class XHRParentComponent {}
@Component(template: const TemplateConfig(inline: 'inline component')) 
class MainComponent {}
@Component(template: const TemplateConfig(inline: 'nested component')) 
class NestedComponent {}
@Component(template: const TemplateConfig(inline: '<div rec-comp></div>'), selector: 'rec-comp') 
class RecursiveComponent {}
class TestableCompiler extends Compiler {
List steps;
TestableCompiler(DirectiveMetadataReader reader, List steps, ShadowDomStrategy strategy, XHRMock xhr): super(dynamicChangeDetection, new TemplateLoader(xhr), reader, new Parser(new Lexer()), new CompilerCache(), strategy) {
  this.steps = steps;
}
List createSteps(component) {
  return this.steps;
}
}
class MockStep extends CompileStep {
Function processClosure;
MockStep(process) {
  this.processClosure = process;
}
process(CompileElement parent, CompileElement current, CompileControl control) {
  this.processClosure(parent, current, control);
}
}
class FakeShadowDomStrategy extends NativeShadowDomStrategy {
String templateHtml;
FakeShadowDomStrategy(String templateHtml) {
  this.templateHtml = templateHtml;
}
processTemplate(Element template, DirectiveMetadata cmpMetadata) {
  DOM.setInnerHTML(template, this.templateHtml);
}
}
